/*******************************************************************************
 * Copyright (c) 2006-2012
 * Software Technology Group, Dresden University of Technology
 * DevBoost GmbH, Berlin, Amtsgericht Charlottenburg, HRB 140026
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Software Technology Group - TU Dresden, Germany;
 *   DevBoost GmbH - Berlin, Germany
 *      - initial API and implementation
 ******************************************************************************/

package org.reuseware.coconut.fracol.resource.fracol.mopp;

public class FracolPrinter implements org.reuseware.coconut.fracol.resource.fracol.IFracolTextPrinter {
	
	protected org.reuseware.coconut.fracol.resource.fracol.IFracolTokenResolverFactory tokenResolverFactory = new org.reuseware.coconut.fracol.resource.fracol.mopp.FracolTokenResolverFactory();
	
	protected java.io.OutputStream outputStream;
	
	/**
	 * Holds the resource that is associated with this printer. This may be null if
	 * the printer is used stand alone.
	 */
	private org.reuseware.coconut.fracol.resource.fracol.IFracolTextResource resource;
	
	private java.util.Map<?, ?> options;
	private String encoding = System.getProperty("file.encoding");
	
	public FracolPrinter(java.io.OutputStream outputStream, org.reuseware.coconut.fracol.resource.fracol.IFracolTextResource resource) {
		super();
		this.outputStream = outputStream;
		this.resource = resource;
	}
	
	protected int matchCount(java.util.Map<String, Integer> featureCounter, java.util.Collection<String> needed) {
		int pos = 0;
		int neg = 0;
		
		for (String featureName : featureCounter.keySet()) {
			if (needed.contains(featureName)) {
				int value = featureCounter.get(featureName);
				if (value == 0) {
					neg += 1;
				} else {
					pos += 1;
				}
			}
		}
		return neg > 0 ? -neg : pos;
	}
	
	protected void doPrint(org.eclipse.emf.ecore.EObject element, java.io.PrintWriter out, String globaltab) {
		if (element == null) {
			throw new java.lang.IllegalArgumentException("Nothing to write.");
		}
		if (out == null) {
			throw new java.lang.IllegalArgumentException("Nothing to write on.");
		}
		
		if (element instanceof org.reuseware.coconut.fracol.FragmentCollaboration) {
			print_org_reuseware_coconut_fracol_FragmentCollaboration((org.reuseware.coconut.fracol.FragmentCollaboration) element, globaltab, out);
			return;
		}
		if (element instanceof org.reuseware.coconut.fracol.FragmentRole) {
			print_org_reuseware_coconut_fracol_FragmentRole((org.reuseware.coconut.fracol.FragmentRole) element, globaltab, out);
			return;
		}
		if (element instanceof org.reuseware.coconut.fracol.StaticPortType) {
			print_org_reuseware_coconut_fracol_StaticPortType((org.reuseware.coconut.fracol.StaticPortType) element, globaltab, out);
			return;
		}
		if (element instanceof org.reuseware.coconut.fracol.DynamicPortType) {
			print_org_reuseware_coconut_fracol_DynamicPortType((org.reuseware.coconut.fracol.DynamicPortType) element, globaltab, out);
			return;
		}
		if (element instanceof org.reuseware.coconut.fracol.Configuration) {
			print_org_reuseware_coconut_fracol_Configuration((org.reuseware.coconut.fracol.Configuration) element, globaltab, out);
			return;
		}
		if (element instanceof org.reuseware.coconut.fracol.Contribution) {
			print_org_reuseware_coconut_fracol_Contribution((org.reuseware.coconut.fracol.Contribution) element, globaltab, out);
			return;
		}
		
		addWarningToResource("The printer can not handle " + element.eClass().getName() + " elements", element);
	}
	
	protected org.reuseware.coconut.fracol.resource.fracol.mopp.FracolReferenceResolverSwitch getReferenceResolverSwitch() {
		return (org.reuseware.coconut.fracol.resource.fracol.mopp.FracolReferenceResolverSwitch) new org.reuseware.coconut.fracol.resource.fracol.mopp.FracolMetaInformation().getReferenceResolverSwitch();
	}
	
	protected void addWarningToResource(final String errorMessage, org.eclipse.emf.ecore.EObject cause) {
		org.reuseware.coconut.fracol.resource.fracol.IFracolTextResource resource = getResource();
		if (resource == null) {
			// the resource can be null if the printer is used stand alone
			return;
		}
		resource.addProblem(new org.reuseware.coconut.fracol.resource.fracol.mopp.FracolProblem(errorMessage, org.reuseware.coconut.fracol.resource.fracol.FracolEProblemType.PRINT_PROBLEM, org.reuseware.coconut.fracol.resource.fracol.FracolEProblemSeverity.WARNING), cause);
	}
	
	public void setOptions(java.util.Map<?,?> options) {
		this.options = options;
	}
	
	public java.util.Map<?,?> getOptions() {
		return options;
	}
	
	public void setEncoding(String encoding) {
		if (encoding != null) {
			this.encoding = encoding;
		}
	}
	
	public String getEncoding() {
		return encoding;
	}
	
	public org.reuseware.coconut.fracol.resource.fracol.IFracolTextResource getResource() {
		return resource;
	}
	
	/**
	 * Calls {@link #doPrint(EObject, PrintWriter, String)} and writes the result to
	 * the underlying output stream.
	 */
	public void print(org.eclipse.emf.ecore.EObject element) throws java.io.IOException {
		java.io.PrintWriter out = new java.io.PrintWriter(new java.io.OutputStreamWriter(new java.io.BufferedOutputStream(outputStream), encoding));
		doPrint(element, out, "");
		out.flush();
		out.close();
	}
	
	public void print_org_reuseware_coconut_fracol_FragmentCollaboration(org.reuseware.coconut.fracol.FragmentCollaboration element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(4);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.FRAGMENT_COLLABORATION__COMPOSITION_ASSOCIATIONS));
		printCountingMap.put("compositionAssociations", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.FRAGMENT_COLLABORATION__FRACOL_NAME));
		printCountingMap.put("fracolName", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.FRAGMENT_COLLABORATION__FRACOL_NAMESPACE));
		printCountingMap.put("fracolNamespace", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.FRAGMENT_COLLABORATION__FRAGMENT_ROLES));
		printCountingMap.put("fragmentRoles", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		// print collected hidden tokens
		int count;
		boolean iterate = true;
		java.io.StringWriter sWriter = null;
		java.io.PrintWriter out1 = null;
		java.util.Map<String, Integer> printCountingMap1 = null;
		// DEFINITION PART BEGINS (CsString)
		out.print("fracol");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		iterate = true;
		while (iterate) {
			sWriter = new java.io.StringWriter();
			out1 = new java.io.PrintWriter(sWriter);
			printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
			print_org_reuseware_coconut_fracol_FragmentCollaboration_0(element, localtab, out1, printCountingMap1);
			if (printCountingMap.equals(printCountingMap1)) {
				iterate = false;
				out1.close();
			} else {
				out1.flush();
				out1.close();
				out.print(sWriter.toString());
				printCountingMap.putAll(printCountingMap1);
			}
		}
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("fracolName");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.FRAGMENT_COLLABORATION__FRACOL_NAME));
			if (o != null) {
				org.reuseware.coconut.fracol.resource.fracol.IFracolTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.FRAGMENT_COLLABORATION__FRACOL_NAME), element));
				out.print(" ");
			}
			printCountingMap.put("fracolName", count - 1);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		iterate = true;
		while (iterate) {
			sWriter = new java.io.StringWriter();
			out1 = new java.io.PrintWriter(sWriter);
			printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
			print_org_reuseware_coconut_fracol_FragmentCollaboration_1(element, localtab, out1, printCountingMap1);
			if (printCountingMap.equals(printCountingMap1)) {
				iterate = false;
				out1.close();
			} else {
				out1.flush();
				out1.close();
				out.print(sWriter.toString());
				printCountingMap.putAll(printCountingMap1);
			}
		}
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
	}
	
	public void print_org_reuseware_coconut_fracol_FragmentCollaboration_0(org.reuseware.coconut.fracol.FragmentCollaboration element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("fracolNamespace");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.FRAGMENT_COLLABORATION__FRACOL_NAMESPACE));
			java.util.List<?> list = (java.util.List<?>) o;
			int index = list.size() - count;
			if (index >= 0) {
				o = list.get(index);
			} else {
				o = null;
			}
			if (o != null) {
				org.reuseware.coconut.fracol.resource.fracol.IFracolTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.FRAGMENT_COLLABORATION__FRACOL_NAMESPACE), element));
				out.print(" ");
			}
			printCountingMap.put("fracolNamespace", count - 1);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print(".");
		out.print(" ");
	}
	
	public void print_org_reuseware_coconut_fracol_FragmentCollaboration_1(org.reuseware.coconut.fracol.FragmentCollaboration element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		int alt = -1;
		alt = 0;
		int matches = 		matchCount(printCountingMap, java.util.Arrays.asList(		"fragmentRoles"		));
		int tempMatchCount;
		tempMatchCount = 		matchCount(printCountingMap, java.util.Arrays.asList(		"compositionAssociations"		));
		if (tempMatchCount > matches) {
			alt = 1;
			matches = tempMatchCount;
		}
		switch(alt) {
			case 1:			{
				// DEFINITION PART BEGINS (Containment)
				count = printCountingMap.get("compositionAssociations");
				if (count > 0) {
					Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.FRAGMENT_COLLABORATION__COMPOSITION_ASSOCIATIONS));
					java.util.List<?> list = (java.util.List<?>) o;
					int index = list.size() - count;
					if (index >= 0) {
						o = list.get(index);
					} else {
						o = null;
					}
					if (o != null) {
						doPrint((org.eclipse.emf.ecore.EObject) o, out, localtab);
					}
					printCountingMap.put("compositionAssociations", count - 1);
				}
			}
			break;
			default:			// DEFINITION PART BEGINS (Containment)
			count = printCountingMap.get("fragmentRoles");
			if (count > 0) {
				Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.FRAGMENT_COLLABORATION__FRAGMENT_ROLES));
				java.util.List<?> list = (java.util.List<?>) o;
				int index = list.size() - count;
				if (index >= 0) {
					o = list.get(index);
				} else {
					o = null;
				}
				if (o != null) {
					doPrint((org.eclipse.emf.ecore.EObject) o, out, localtab);
				}
				printCountingMap.put("fragmentRoles", count - 1);
			}
		}
	}
	
	
	public void print_org_reuseware_coconut_fracol_FragmentRole(org.reuseware.coconut.fracol.FragmentRole element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(3);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.FRAGMENT_ROLE__FRAGMENT_COLLABORATION));
		printCountingMap.put("fragmentCollaboration", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.FRAGMENT_ROLE__NAME));
		printCountingMap.put("name", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.FRAGMENT_ROLE__PORT_TYPES));
		printCountingMap.put("portTypes", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		// print collected hidden tokens
		int count;
		boolean iterate = true;
		java.io.StringWriter sWriter = null;
		java.io.PrintWriter out1 = null;
		java.util.Map<String, Integer> printCountingMap1 = null;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("fragment");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("role");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("name");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.FRAGMENT_ROLE__NAME));
			if (o != null) {
				org.reuseware.coconut.fracol.resource.fracol.IFracolTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.FRAGMENT_ROLE__NAME), element));
			}
			printCountingMap.put("name", count - 1);
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		iterate = true;
		while (iterate) {
			sWriter = new java.io.StringWriter();
			out1 = new java.io.PrintWriter(sWriter);
			printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
			print_org_reuseware_coconut_fracol_FragmentRole_0(element, localtab, out1, printCountingMap1);
			if (printCountingMap.equals(printCountingMap1)) {
				iterate = false;
				out1.close();
			} else {
				out1.flush();
				out1.close();
				out.print(sWriter.toString());
				printCountingMap.putAll(printCountingMap1);
			}
		}
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
	}
	
	public void print_org_reuseware_coconut_fracol_FragmentRole_0(org.reuseware.coconut.fracol.FragmentRole element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (Containment)
		count = printCountingMap.get("portTypes");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.FRAGMENT_ROLE__PORT_TYPES));
			java.util.List<?> list = (java.util.List<?>) o;
			int index = list.size() - count;
			if (index >= 0) {
				o = list.get(index);
			} else {
				o = null;
			}
			if (o != null) {
				doPrint((org.eclipse.emf.ecore.EObject) o, out, localtab);
			}
			printCountingMap.put("portTypes", count - 1);
		}
	}
	
	
	public void print_org_reuseware_coconut_fracol_StaticPortType(org.reuseware.coconut.fracol.StaticPortType element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(4);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.STATIC_PORT_TYPE__COMPOSITION_ASSOCIATIONS2));
		printCountingMap.put("compositionAssociations2", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.STATIC_PORT_TYPE__COMPOSITION_ASSOCIATIONS));
		printCountingMap.put("compositionAssociations", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.STATIC_PORT_TYPE__FRAGMENT_ROLE));
		printCountingMap.put("fragmentRole", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.STATIC_PORT_TYPE__NAME));
		printCountingMap.put("name", temp == null ? 0 : 1);
		// print collected hidden tokens
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("static");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("port");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("type");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("name");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.STATIC_PORT_TYPE__NAME));
			if (o != null) {
				org.reuseware.coconut.fracol.resource.fracol.IFracolTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.STATIC_PORT_TYPE__NAME), element));
				out.print(" ");
			}
			printCountingMap.put("name", count - 1);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print(";");
		out.print(" ");
	}
	
	
	public void print_org_reuseware_coconut_fracol_DynamicPortType(org.reuseware.coconut.fracol.DynamicPortType element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(4);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.DYNAMIC_PORT_TYPE__COMPOSITION_ASSOCIATIONS2));
		printCountingMap.put("compositionAssociations2", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.DYNAMIC_PORT_TYPE__COMPOSITION_ASSOCIATIONS));
		printCountingMap.put("compositionAssociations", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.DYNAMIC_PORT_TYPE__FRAGMENT_ROLE));
		printCountingMap.put("fragmentRole", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.DYNAMIC_PORT_TYPE__NAME));
		printCountingMap.put("name", temp == null ? 0 : 1);
		// print collected hidden tokens
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("dynamic");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("port");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("type");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("name");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.DYNAMIC_PORT_TYPE__NAME));
			if (o != null) {
				org.reuseware.coconut.fracol.resource.fracol.IFracolTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.DYNAMIC_PORT_TYPE__NAME), element));
				out.print(" ");
			}
			printCountingMap.put("name", count - 1);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print(";");
		out.print(" ");
	}
	
	
	public void print_org_reuseware_coconut_fracol_Configuration(org.reuseware.coconut.fracol.Configuration element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(6);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONFIGURATION__END1_ROLE));
		printCountingMap.put("end1Role", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONFIGURATION__END1));
		printCountingMap.put("end1", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONFIGURATION__END2_ROLE));
		printCountingMap.put("end2Role", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONFIGURATION__END2));
		printCountingMap.put("end2", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONFIGURATION__FRAGMENT_COLLABORATION));
		printCountingMap.put("fragmentCollaboration", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONFIGURATION__NAME));
		printCountingMap.put("name", temp == null ? 0 : 1);
		// print collected hidden tokens
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("configuring");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("association");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("name");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONFIGURATION__NAME));
			if (o != null) {
				org.reuseware.coconut.fracol.resource.fracol.IFracolTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONFIGURATION__NAME), element));
			}
			printCountingMap.put("name", count - 1);
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("end1Role");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONFIGURATION__END1_ROLE));
			if (o != null) {
				org.reuseware.coconut.fracol.resource.fracol.IFracolTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getCompositionAssociationEnd1RoleReferenceResolver().deResolve((org.reuseware.coconut.fracol.FragmentRole) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONFIGURATION__END1_ROLE)), element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONFIGURATION__END1_ROLE), element));
				out.print(" ");
			}
			printCountingMap.put("end1Role", count - 1);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print(".");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("end1");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONFIGURATION__END1));
			if (o != null) {
				org.reuseware.coconut.fracol.resource.fracol.IFracolTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getCompositionAssociationEnd1ReferenceResolver().deResolve((org.reuseware.coconut.fracol.PortType) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONFIGURATION__END1)), element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONFIGURATION__END1), element));
			}
			printCountingMap.put("end1", count - 1);
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("-->");
		out.print(" ");
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("end2Role");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONFIGURATION__END2_ROLE));
			if (o != null) {
				org.reuseware.coconut.fracol.resource.fracol.IFracolTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getCompositionAssociationEnd2RoleReferenceResolver().deResolve((org.reuseware.coconut.fracol.FragmentRole) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONFIGURATION__END2_ROLE)), element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONFIGURATION__END2_ROLE), element));
				out.print(" ");
			}
			printCountingMap.put("end2Role", count - 1);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print(".");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("end2");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONFIGURATION__END2));
			if (o != null) {
				org.reuseware.coconut.fracol.resource.fracol.IFracolTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getCompositionAssociationEnd2ReferenceResolver().deResolve((org.reuseware.coconut.fracol.PortType) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONFIGURATION__END2)), element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONFIGURATION__END2), element));
				out.print(" ");
			}
			printCountingMap.put("end2", count - 1);
		}
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
	}
	
	
	public void print_org_reuseware_coconut_fracol_Contribution(org.reuseware.coconut.fracol.Contribution element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(6);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONTRIBUTION__END1_ROLE));
		printCountingMap.put("end1Role", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONTRIBUTION__END1));
		printCountingMap.put("end1", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONTRIBUTION__END2_ROLE));
		printCountingMap.put("end2Role", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONTRIBUTION__END2));
		printCountingMap.put("end2", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONTRIBUTION__FRAGMENT_COLLABORATION));
		printCountingMap.put("fragmentCollaboration", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONTRIBUTION__NAME));
		printCountingMap.put("name", temp == null ? 0 : 1);
		// print collected hidden tokens
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("contributing");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("association");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("name");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONTRIBUTION__NAME));
			if (o != null) {
				org.reuseware.coconut.fracol.resource.fracol.IFracolTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONTRIBUTION__NAME), element));
			}
			printCountingMap.put("name", count - 1);
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("end1Role");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONTRIBUTION__END1_ROLE));
			if (o != null) {
				org.reuseware.coconut.fracol.resource.fracol.IFracolTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getCompositionAssociationEnd1RoleReferenceResolver().deResolve((org.reuseware.coconut.fracol.FragmentRole) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONTRIBUTION__END1_ROLE)), element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONTRIBUTION__END1_ROLE), element));
				out.print(" ");
			}
			printCountingMap.put("end1Role", count - 1);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print(".");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("end1");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONTRIBUTION__END1));
			if (o != null) {
				org.reuseware.coconut.fracol.resource.fracol.IFracolTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getCompositionAssociationEnd1ReferenceResolver().deResolve((org.reuseware.coconut.fracol.PortType) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONTRIBUTION__END1)), element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONTRIBUTION__END1), element));
			}
			printCountingMap.put("end1", count - 1);
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("-->");
		out.print(" ");
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("end2Role");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONTRIBUTION__END2_ROLE));
			if (o != null) {
				org.reuseware.coconut.fracol.resource.fracol.IFracolTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getCompositionAssociationEnd2RoleReferenceResolver().deResolve((org.reuseware.coconut.fracol.FragmentRole) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONTRIBUTION__END2_ROLE)), element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONTRIBUTION__END2_ROLE), element));
				out.print(" ");
			}
			printCountingMap.put("end2Role", count - 1);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print(".");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("end2");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONTRIBUTION__END2));
			if (o != null) {
				org.reuseware.coconut.fracol.resource.fracol.IFracolTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getCompositionAssociationEnd2ReferenceResolver().deResolve((org.reuseware.coconut.fracol.PortType) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONTRIBUTION__END2)), element.eClass().getEStructuralFeature(org.reuseware.coconut.fracol.FracolPackage.CONTRIBUTION__END2), element));
				out.print(" ");
			}
			printCountingMap.put("end2", count - 1);
		}
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
	}
	
	
}
