/*******************************************************************************
 * Copyright (c) 2006-2012
 * Software Technology Group, Dresden University of Technology
 * DevBoost GmbH, Berlin, Amtsgericht Charlottenburg, HRB 140026
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Software Technology Group - TU Dresden, Germany;
 *   DevBoost GmbH - Berlin, Germany
 *      - initial API and implementation
 ******************************************************************************/

package org.reuseware.coconut.fracol.resource.fracol.mopp;

public class FracolResource extends org.eclipse.emf.ecore.resource.impl.ResourceImpl implements org.reuseware.coconut.fracol.resource.fracol.IFracolTextResource {
	
	public class ElementBasedTextDiagnostic implements org.reuseware.coconut.fracol.resource.fracol.IFracolTextDiagnostic {
		
		private final org.reuseware.coconut.fracol.resource.fracol.IFracolLocationMap locationMap;
		private final org.eclipse.emf.common.util.URI uri;
		private final org.eclipse.emf.ecore.EObject element;
		private final org.reuseware.coconut.fracol.resource.fracol.IFracolProblem problem;
		
		public ElementBasedTextDiagnostic(org.reuseware.coconut.fracol.resource.fracol.IFracolLocationMap locationMap, org.eclipse.emf.common.util.URI uri, org.reuseware.coconut.fracol.resource.fracol.IFracolProblem problem, org.eclipse.emf.ecore.EObject element) {
			super();
			this.uri = uri;
			this.locationMap = locationMap;
			this.element = element;
			this.problem = problem;
		}
		
		public String getMessage() {
			return problem.getMessage();
		}
		
		public org.reuseware.coconut.fracol.resource.fracol.IFracolProblem getProblem() {
			return problem;
		}
		
		public String getLocation() {
			return uri.toString();
		}
		
		public int getCharStart() {
			return Math.max(0, locationMap.getCharStart(element));
		}
		
		public int getCharEnd() {
			return Math.max(0, locationMap.getCharEnd(element));
		}
		
		public int getColumn() {
			return Math.max(0, locationMap.getColumn(element));
		}
		
		public int getLine() {
			return Math.max(0, locationMap.getLine(element));
		}
		
		public org.eclipse.emf.ecore.EObject getElement() {
			return element;
		}
		
		public boolean wasCausedBy(org.eclipse.emf.ecore.EObject element) {
			if (this.element == null) {
				return false;
			}
			return this.element.equals(element);
		}
		
		public String toString() {
			return getMessage() + " at " + getLocation() + " line " + getLine() + ", column " + getColumn();
		}
	}
	
	public class PositionBasedTextDiagnostic implements org.reuseware.coconut.fracol.resource.fracol.IFracolTextDiagnostic {
		
		private final org.eclipse.emf.common.util.URI uri;
		
		private int column;
		private int line;
		private int charStart;
		private int charEnd;
		private org.reuseware.coconut.fracol.resource.fracol.IFracolProblem problem;
		
		public PositionBasedTextDiagnostic(org.eclipse.emf.common.util.URI uri, org.reuseware.coconut.fracol.resource.fracol.IFracolProblem problem, int column, int line, int charStart, int charEnd) {
			
			super();
			this.uri = uri;
			this.column = column;
			this.line = line;
			this.charStart = charStart;
			this.charEnd = charEnd;
			this.problem = problem;
		}
		
		public org.reuseware.coconut.fracol.resource.fracol.IFracolProblem getProblem() {
			return problem;
		}
		
		public int getCharStart() {
			return charStart;
		}
		
		public int getCharEnd() {
			return charEnd;
		}
		
		public int getColumn() {
			return column;
		}
		
		public int getLine() {
			return line;
		}
		
		public String getLocation() {
			return uri.toString();
		}
		
		public String getMessage() {
			return problem.getMessage();
		}
		
		public boolean wasCausedBy(org.eclipse.emf.ecore.EObject element) {
			return false;
		}
		
		public String toString() {
			return getMessage() + " at " + getLocation() + " line " + getLine() + ", column " + getColumn();
		}
	}
	
	private org.reuseware.coconut.fracol.resource.fracol.IFracolReferenceResolverSwitch resolverSwitch;
	private org.reuseware.coconut.fracol.resource.fracol.IFracolLocationMap locationMap;
	private int proxyCounter = 0;
	private org.reuseware.coconut.fracol.resource.fracol.IFracolTextParser parser;
	private org.reuseware.coconut.fracol.resource.fracol.util.FracolLayoutUtil layoutUtil = new org.reuseware.coconut.fracol.resource.fracol.util.FracolLayoutUtil();
	private org.reuseware.coconut.fracol.resource.fracol.mopp.FracolMarkerHelper markerHelper;
	private java.util.Map<String, org.reuseware.coconut.fracol.resource.fracol.IFracolContextDependentURIFragment<? extends org.eclipse.emf.ecore.EObject>> internalURIFragmentMap = new java.util.LinkedHashMap<String, org.reuseware.coconut.fracol.resource.fracol.IFracolContextDependentURIFragment<? extends org.eclipse.emf.ecore.EObject>>();
	private java.util.Map<String, org.reuseware.coconut.fracol.resource.fracol.IFracolQuickFix> quickFixMap = new java.util.LinkedHashMap<String, org.reuseware.coconut.fracol.resource.fracol.IFracolQuickFix>();
	private java.util.Map<?, ?> loadOptions;
	
	/**
	 * If a post-processor is currently running, this field holds a reference to it.
	 * This reference is used to terminate post-processing if needed.
	 */
	private org.reuseware.coconut.fracol.resource.fracol.IFracolResourcePostProcessor runningPostProcessor;
	
	/**
	 * A flag (and lock) to indicate whether reloading of the resource shall be
	 * cancelled.
	 */
	private Boolean terminateReload = false;
	private Object terminateReloadLock = new Object();
	private Object loadingLock = new Object();
	private boolean delayNotifications = false;
	private java.util.List<org.eclipse.emf.common.notify.Notification> delayedNotifications = new java.util.ArrayList<org.eclipse.emf.common.notify.Notification>();
	private java.io.InputStream latestReloadInputStream = null;
	private java.util.Map<?, ?> latestReloadOptions = null;
	private org.reuseware.coconut.fracol.resource.fracol.util.FracolInterruptibleEcoreResolver interruptibleResolver;
	
	protected org.reuseware.coconut.fracol.resource.fracol.mopp.FracolMetaInformation metaInformation = new org.reuseware.coconut.fracol.resource.fracol.mopp.FracolMetaInformation();
	
	public FracolResource() {
		super();
		resetLocationMap();
	}
	
	public FracolResource(org.eclipse.emf.common.util.URI uri) {
		super(uri);
		resetLocationMap();
	}
	
	protected void doLoad(java.io.InputStream inputStream, java.util.Map<?,?> options) throws java.io.IOException {
		synchronized (loadingLock) {
			if (processTerminationRequested()) {
				return;
			}
			this.loadOptions = options;
			delayNotifications = true;
			resetLocationMap();
			String encoding = getEncoding(options);
			java.io.InputStream actualInputStream = inputStream;
			Object inputStreamPreProcessorProvider = null;
			if (options != null) {
				inputStreamPreProcessorProvider = options.get(org.reuseware.coconut.fracol.resource.fracol.IFracolOptions.INPUT_STREAM_PREPROCESSOR_PROVIDER);
			}
			if (inputStreamPreProcessorProvider != null) {
				if (inputStreamPreProcessorProvider instanceof org.reuseware.coconut.fracol.resource.fracol.IFracolInputStreamProcessorProvider) {
					org.reuseware.coconut.fracol.resource.fracol.IFracolInputStreamProcessorProvider provider = (org.reuseware.coconut.fracol.resource.fracol.IFracolInputStreamProcessorProvider) inputStreamPreProcessorProvider;
					org.reuseware.coconut.fracol.resource.fracol.mopp.FracolInputStreamProcessor processor = provider.getInputStreamProcessor(inputStream);
					actualInputStream = processor;
				}
			}
			
			parser = getMetaInformation().createParser(actualInputStream, encoding);
			parser.setOptions(options);
			org.reuseware.coconut.fracol.resource.fracol.IFracolReferenceResolverSwitch referenceResolverSwitch = getReferenceResolverSwitch();
			referenceResolverSwitch.setOptions(options);
			org.reuseware.coconut.fracol.resource.fracol.IFracolParseResult result = parser.parse();
			// dispose parser, we don't need it anymore
			parser = null;
			
			if (processTerminationRequested()) {
				// do nothing if reload was already restarted
				return;
			}
			
			clearState();
			getContentsInternal().clear();
			org.eclipse.emf.ecore.EObject root = null;
			if (result != null) {
				root = result.getRoot();
				if (root != null) {
					if (isLayoutInformationRecordingEnabled()) {
						layoutUtil.transferAllLayoutInformationToModel(root);
					}
					if (processTerminationRequested()) {
						// the next reload will add new content
						return;
					}
					getContentsInternal().add(root);
				}
				java.util.Collection<org.reuseware.coconut.fracol.resource.fracol.IFracolCommand<org.reuseware.coconut.fracol.resource.fracol.IFracolTextResource>> commands = result.getPostParseCommands();
				if (commands != null) {
					for (org.reuseware.coconut.fracol.resource.fracol.IFracolCommand<org.reuseware.coconut.fracol.resource.fracol.IFracolTextResource>  command : commands) {
						command.execute(this);
					}
				}
			}
			getReferenceResolverSwitch().setOptions(options);
			if (getErrors().isEmpty()) {
				if (!runPostProcessors(options)) {
					return;
				}
				if (root != null) {
					runValidators(root);
				}
			}
			notifyDelayed();
		}
	}
	
	protected boolean processTerminationRequested() {
		if (terminateReload) {
			delayNotifications = false;
			delayedNotifications.clear();
			return true;
		}
		return false;
	}
	protected void notifyDelayed() {
		delayNotifications = false;
		for (org.eclipse.emf.common.notify.Notification delayedNotification : delayedNotifications) {
			super.eNotify(delayedNotification);
		}
		delayedNotifications.clear();
	}
	public void eNotify(org.eclipse.emf.common.notify.Notification notification) {
		if (delayNotifications) {
			delayedNotifications.add(notification);
		} else {
			super.eNotify(notification);
		}
	}
	/**
	 * Reloads the contents of this resource from the given stream.
	 */
	public void reload(java.io.InputStream inputStream, java.util.Map<?,?> options) throws java.io.IOException {
		synchronized (terminateReloadLock) {
			latestReloadInputStream = inputStream;
			latestReloadOptions = options;
			if (terminateReload == true) {
				// //reload already requested
			}
			terminateReload = true;
		}
		cancelReload();
		synchronized (loadingLock) {
			synchronized (terminateReloadLock) {
				terminateReload = false;
			}
			isLoaded = false;
			java.util.Map<Object, Object> loadOptions = addDefaultLoadOptions(latestReloadOptions);
			try {
				doLoad(latestReloadInputStream, loadOptions);
			} catch (org.reuseware.coconut.fracol.resource.fracol.mopp.FracolTerminateParsingException tpe) {
				// do nothing - the resource is left unchanged if this exception is thrown
			}
			resolveAfterParsing();
			isLoaded = true;
		}
	}
	
	/**
	 * Cancels reloading this resource. The running parser and post processors are
	 * terminated.
	 */
	protected void cancelReload() {
		// Cancel parser
		org.reuseware.coconut.fracol.resource.fracol.IFracolTextParser parserCopy = parser;
		if (parserCopy != null) {
			parserCopy.terminate();
		}
		// Cancel post processor(s)
		org.reuseware.coconut.fracol.resource.fracol.IFracolResourcePostProcessor runningPostProcessorCopy = runningPostProcessor;
		if (runningPostProcessorCopy != null) {
			runningPostProcessorCopy.terminate();
		}
		// Cancel reference resolving
		org.reuseware.coconut.fracol.resource.fracol.util.FracolInterruptibleEcoreResolver interruptibleResolverCopy = interruptibleResolver;
		if (interruptibleResolverCopy != null) {
			interruptibleResolverCopy.terminate();
		}
	}
	
	protected void doSave(java.io.OutputStream outputStream, java.util.Map<?,?> options) throws java.io.IOException {
		org.reuseware.coconut.fracol.resource.fracol.IFracolTextPrinter printer = getMetaInformation().createPrinter(outputStream, this);
		org.reuseware.coconut.fracol.resource.fracol.IFracolReferenceResolverSwitch referenceResolverSwitch = getReferenceResolverSwitch();
		printer.setEncoding(getEncoding(options));
		referenceResolverSwitch.setOptions(options);
		for (org.eclipse.emf.ecore.EObject root : getContentsInternal()) {
			if (isLayoutInformationRecordingEnabled()) {
				layoutUtil.transferAllLayoutInformationFromModel(root);
			}
			printer.print(root);
			if (isLayoutInformationRecordingEnabled()) {
				layoutUtil.transferAllLayoutInformationToModel(root);
			}
		}
	}
	
	protected String getSyntaxName() {
		return "fracol";
	}
	
	public String getEncoding(java.util.Map<?, ?> options) {
		String encoding = null;
		if (new org.reuseware.coconut.fracol.resource.fracol.util.FracolRuntimeUtil().isEclipsePlatformAvailable()) {
			encoding = new org.reuseware.coconut.fracol.resource.fracol.util.FracolEclipseProxy().getPlatformResourceEncoding(uri);
		}
		if (options != null) {
			Object encodingOption = options.get(org.reuseware.coconut.fracol.resource.fracol.IFracolOptions.OPTION_ENCODING);
			if (encodingOption != null) {
				encoding = encodingOption.toString();
			}
		}
		return encoding;
	}
	
	public org.reuseware.coconut.fracol.resource.fracol.IFracolReferenceResolverSwitch getReferenceResolverSwitch() {
		if (resolverSwitch == null) {
			resolverSwitch = new org.reuseware.coconut.fracol.resource.fracol.mopp.FracolReferenceResolverSwitch();
		}
		return resolverSwitch;
	}
	
	public org.reuseware.coconut.fracol.resource.fracol.mopp.FracolMetaInformation getMetaInformation() {
		return new org.reuseware.coconut.fracol.resource.fracol.mopp.FracolMetaInformation();
	}
	
	/**
	 * Clears the location map by replacing it with a new instance.
	 */
	protected void resetLocationMap() {
		if (isLocationMapEnabled()) {
			locationMap = new org.reuseware.coconut.fracol.resource.fracol.mopp.FracolLocationMap();
		} else {
			locationMap = new org.reuseware.coconut.fracol.resource.fracol.mopp.FracolDevNullLocationMap();
		}
	}
	
	public void addURIFragment(String internalURIFragment, org.reuseware.coconut.fracol.resource.fracol.IFracolContextDependentURIFragment<? extends org.eclipse.emf.ecore.EObject> uriFragment) {
		internalURIFragmentMap.put(internalURIFragment, uriFragment);
	}
	
	public <ContainerType extends org.eclipse.emf.ecore.EObject, ReferenceType extends org.eclipse.emf.ecore.EObject> void registerContextDependentProxy(org.reuseware.coconut.fracol.resource.fracol.IFracolContextDependentURIFragmentFactory<ContainerType, ReferenceType> factory, ContainerType container, org.eclipse.emf.ecore.EReference reference, String id, org.eclipse.emf.ecore.EObject proxyElement, int position) {
		org.eclipse.emf.ecore.InternalEObject proxy = (org.eclipse.emf.ecore.InternalEObject) proxyElement;
		String internalURIFragment = org.reuseware.coconut.fracol.resource.fracol.IFracolContextDependentURIFragment.INTERNAL_URI_FRAGMENT_PREFIX + (proxyCounter++) + "_" + id;
		org.reuseware.coconut.fracol.resource.fracol.IFracolContextDependentURIFragment<?> uriFragment = factory.create(id, container, reference, position, proxy);
		proxy.eSetProxyURI(getURI().appendFragment(internalURIFragment));
		addURIFragment(internalURIFragment, uriFragment);
	}
	
	public org.eclipse.emf.ecore.EObject getEObject(String id) {
		if (internalURIFragmentMap.containsKey(id)) {
			org.reuseware.coconut.fracol.resource.fracol.IFracolContextDependentURIFragment<? extends org.eclipse.emf.ecore.EObject> uriFragment = internalURIFragmentMap.get(id);
			boolean wasResolvedBefore = uriFragment.isResolved();
			org.reuseware.coconut.fracol.resource.fracol.IFracolReferenceResolveResult<? extends org.eclipse.emf.ecore.EObject> result = null;
			// catch and report all Exceptions that occur during proxy resolving
			try {
				result = uriFragment.resolve();
			} catch (Exception e) {
				String message = "An expection occured while resolving the proxy for: "+ id + ". (" + e.toString() + ")";
				addProblem(new org.reuseware.coconut.fracol.resource.fracol.mopp.FracolProblem(message, org.reuseware.coconut.fracol.resource.fracol.FracolEProblemType.UNRESOLVED_REFERENCE, org.reuseware.coconut.fracol.resource.fracol.FracolEProblemSeverity.ERROR), uriFragment.getProxy());
				new org.reuseware.coconut.fracol.resource.fracol.util.FracolRuntimeUtil().logError(message, e);
			}
			if (result == null) {
				// the resolving did call itself
				return null;
			}
			if (!wasResolvedBefore && !result.wasResolved()) {
				attachResolveError(result, uriFragment.getProxy());
				return null;
			} else if (!result.wasResolved()) {
				return null;
			} else {
				org.eclipse.emf.ecore.EObject proxy = uriFragment.getProxy();
				// remove an error that might have been added by an earlier attempt
				removeDiagnostics(proxy, getErrors());
				// remove old warnings and attach new
				removeDiagnostics(proxy, getWarnings());
				attachResolveWarnings(result, proxy);
				org.reuseware.coconut.fracol.resource.fracol.IFracolReferenceMapping<? extends org.eclipse.emf.ecore.EObject> mapping = result.getMappings().iterator().next();
				org.eclipse.emf.ecore.EObject resultElement = getResultElement(uriFragment, mapping, proxy, result.getErrorMessage());
				org.eclipse.emf.ecore.EObject container = uriFragment.getContainer();
				replaceProxyInLayoutAdapters(container, proxy, resultElement);
				return resultElement;
			}
		} else {
			return super.getEObject(id);
		}
	}
	
	protected void replaceProxyInLayoutAdapters(org.eclipse.emf.ecore.EObject container, org.eclipse.emf.ecore.EObject proxy, org.eclipse.emf.ecore.EObject target) {
		for (org.eclipse.emf.common.notify.Adapter adapter : container.eAdapters()) {
			if (adapter instanceof org.reuseware.coconut.fracol.resource.fracol.mopp.FracolLayoutInformationAdapter) {
				org.reuseware.coconut.fracol.resource.fracol.mopp.FracolLayoutInformationAdapter layoutInformationAdapter = (org.reuseware.coconut.fracol.resource.fracol.mopp.FracolLayoutInformationAdapter) adapter;
				layoutInformationAdapter.replaceProxy(proxy, target);
			}
		}
	}
	
	protected org.eclipse.emf.ecore.EObject getResultElement(org.reuseware.coconut.fracol.resource.fracol.IFracolContextDependentURIFragment<? extends org.eclipse.emf.ecore.EObject> uriFragment, org.reuseware.coconut.fracol.resource.fracol.IFracolReferenceMapping<? extends org.eclipse.emf.ecore.EObject> mapping, org.eclipse.emf.ecore.EObject proxy, final String errorMessage) {
		if (mapping instanceof org.reuseware.coconut.fracol.resource.fracol.IFracolURIMapping<?>) {
			org.eclipse.emf.common.util.URI uri = ((org.reuseware.coconut.fracol.resource.fracol.IFracolURIMapping<? extends org.eclipse.emf.ecore.EObject>)mapping).getTargetIdentifier();
			if (uri != null) {
				org.eclipse.emf.ecore.EObject result = null;
				try {
					result = this.getResourceSet().getEObject(uri, true);
				} catch (Exception e) {
					// we can catch exceptions here, because EMF will try to resolve again and handle
					// the exception
				}
				if (result == null || result.eIsProxy()) {
					// unable to resolve: attach error
					if (errorMessage == null) {
						assert(false);
					} else {
						addProblem(new org.reuseware.coconut.fracol.resource.fracol.mopp.FracolProblem(errorMessage, org.reuseware.coconut.fracol.resource.fracol.FracolEProblemType.UNRESOLVED_REFERENCE, org.reuseware.coconut.fracol.resource.fracol.FracolEProblemSeverity.ERROR), proxy);
					}
				}
				return result;
			}
			return null;
		} else if (mapping instanceof org.reuseware.coconut.fracol.resource.fracol.IFracolElementMapping<?>) {
			org.eclipse.emf.ecore.EObject element = ((org.reuseware.coconut.fracol.resource.fracol.IFracolElementMapping<? extends org.eclipse.emf.ecore.EObject>)mapping).getTargetElement();
			org.eclipse.emf.ecore.EReference reference = uriFragment.getReference();
			org.eclipse.emf.ecore.EReference oppositeReference = uriFragment.getReference().getEOpposite();
			if (!uriFragment.getReference().isContainment() && oppositeReference != null) {
				if (reference.isMany()) {
					org.eclipse.emf.ecore.util.EObjectWithInverseResolvingEList.ManyInverse<org.eclipse.emf.ecore.EObject> list = org.reuseware.coconut.fracol.resource.fracol.util.FracolCastUtil.cast(element.eGet(oppositeReference, false));										// avoids duplicate entries in the reference caused by adding to the
					// oppositeReference
					list.basicAdd(uriFragment.getContainer(),null);
				} else {
					uriFragment.getContainer().eSet(uriFragment.getReference(), element);
				}
			}
			return element;
		} else {
			assert(false);
			return null;
		}
	}
	
	protected void removeDiagnostics(org.eclipse.emf.ecore.EObject cause, java.util.List<org.eclipse.emf.ecore.resource.Resource.Diagnostic> diagnostics) {
		// remove all errors/warnings from this resource
		for (org.eclipse.emf.ecore.resource.Resource.Diagnostic errorCand : new org.eclipse.emf.common.util.BasicEList<org.eclipse.emf.ecore.resource.Resource.Diagnostic>(diagnostics)) {
			if (errorCand instanceof org.reuseware.coconut.fracol.resource.fracol.IFracolTextDiagnostic) {
				if (((org.reuseware.coconut.fracol.resource.fracol.IFracolTextDiagnostic) errorCand).wasCausedBy(cause)) {
					diagnostics.remove(errorCand);
					unmark(cause);
				}
			}
		}
	}
	
	protected void attachResolveError(org.reuseware.coconut.fracol.resource.fracol.IFracolReferenceResolveResult<?> result, org.eclipse.emf.ecore.EObject proxy) {
		// attach errors to this resource
		assert result != null;
		final String errorMessage = result.getErrorMessage();
		if (errorMessage == null) {
			assert(false);
		} else {
			addProblem(new org.reuseware.coconut.fracol.resource.fracol.mopp.FracolProblem(errorMessage, org.reuseware.coconut.fracol.resource.fracol.FracolEProblemType.UNRESOLVED_REFERENCE, org.reuseware.coconut.fracol.resource.fracol.FracolEProblemSeverity.ERROR, result.getQuickFixes()), proxy);
		}
	}
	
	protected void attachResolveWarnings(org.reuseware.coconut.fracol.resource.fracol.IFracolReferenceResolveResult<? extends org.eclipse.emf.ecore.EObject> result, org.eclipse.emf.ecore.EObject proxy) {
		assert result != null;
		assert result.wasResolved();
		if (result.wasResolved()) {
			for (org.reuseware.coconut.fracol.resource.fracol.IFracolReferenceMapping<? extends org.eclipse.emf.ecore.EObject> mapping : result.getMappings()) {
				final String warningMessage = mapping.getWarning();
				if (warningMessage == null) {
					continue;
				}
				addProblem(new org.reuseware.coconut.fracol.resource.fracol.mopp.FracolProblem(warningMessage, org.reuseware.coconut.fracol.resource.fracol.FracolEProblemType.UNRESOLVED_REFERENCE, org.reuseware.coconut.fracol.resource.fracol.FracolEProblemSeverity.WARNING), proxy);
			}
		}
	}
	
	/**
	 * Extends the super implementation by clearing all information about element
	 * positions.
	 */
	protected void doUnload() {
		super.doUnload();
		clearState();
		loadOptions = null;
	}
	
	/**
	 * Runs all post processors to process this resource.
	 */
	protected boolean runPostProcessors(java.util.Map<?, ?> loadOptions) {
		unmark(org.reuseware.coconut.fracol.resource.fracol.FracolEProblemType.ANALYSIS_PROBLEM);
		if (processTerminationRequested()) {
			return false;
		}
		// first, run the generated post processor
		runPostProcessor(new org.reuseware.coconut.fracol.resource.fracol.mopp.FracolResourcePostProcessor());
		if (loadOptions == null) {
			return true;
		}
		// then, run post processors that are registered via the load options extension
		// point
		Object resourcePostProcessorProvider = loadOptions.get(org.reuseware.coconut.fracol.resource.fracol.IFracolOptions.RESOURCE_POSTPROCESSOR_PROVIDER);
		if (resourcePostProcessorProvider != null) {
			if (resourcePostProcessorProvider instanceof org.reuseware.coconut.fracol.resource.fracol.IFracolResourcePostProcessorProvider) {
				runPostProcessor(((org.reuseware.coconut.fracol.resource.fracol.IFracolResourcePostProcessorProvider) resourcePostProcessorProvider).getResourcePostProcessor());
			} else if (resourcePostProcessorProvider instanceof java.util.Collection<?>) {
				java.util.Collection<?> resourcePostProcessorProviderCollection = (java.util.Collection<?>) resourcePostProcessorProvider;
				for (Object processorProvider : resourcePostProcessorProviderCollection) {
					if (processTerminationRequested()) {
						return false;
					}
					if (processorProvider instanceof org.reuseware.coconut.fracol.resource.fracol.IFracolResourcePostProcessorProvider) {
						org.reuseware.coconut.fracol.resource.fracol.IFracolResourcePostProcessorProvider csProcessorProvider = (org.reuseware.coconut.fracol.resource.fracol.IFracolResourcePostProcessorProvider) processorProvider;
						org.reuseware.coconut.fracol.resource.fracol.IFracolResourcePostProcessor postProcessor = csProcessorProvider.getResourcePostProcessor();
						runPostProcessor(postProcessor);
					}
				}
			}
		}
		return true;
	}
	
	/**
	 * Runs the given post processor to process this resource.
	 */
	protected void runPostProcessor(org.reuseware.coconut.fracol.resource.fracol.IFracolResourcePostProcessor postProcessor) {
		try {
			this.runningPostProcessor = postProcessor;
			postProcessor.process(this);
		} catch (Exception e) {
			new org.reuseware.coconut.fracol.resource.fracol.util.FracolRuntimeUtil().logError("Exception while running a post-processor.", e);
		}
		this.runningPostProcessor = null;
	}
	
	public void load(java.util.Map<?, ?> options) throws java.io.IOException {
		java.util.Map<Object, Object> loadOptions = addDefaultLoadOptions(options);
		super.load(loadOptions);
		resolveAfterParsing();
	}
	
	protected void resolveAfterParsing() {
		interruptibleResolver = new org.reuseware.coconut.fracol.resource.fracol.util.FracolInterruptibleEcoreResolver();
		interruptibleResolver.resolveAll(this);
		interruptibleResolver = null;
	}
	
	public void setURI(org.eclipse.emf.common.util.URI uri) {
		// because of the context dependent proxy resolving it is essential to resolve all
		// proxies before the URI is changed which can cause loss of object identities
		org.eclipse.emf.ecore.util.EcoreUtil.resolveAll(this);
		super.setURI(uri);
	}
	
	/**
	 * Returns the location map that contains information about the position of the
	 * contents of this resource in the original textual representation.
	 */
	public org.reuseware.coconut.fracol.resource.fracol.IFracolLocationMap getLocationMap() {
		return locationMap;
	}
	
	public void addProblem(org.reuseware.coconut.fracol.resource.fracol.IFracolProblem problem, org.eclipse.emf.ecore.EObject element) {
		ElementBasedTextDiagnostic diagnostic = new ElementBasedTextDiagnostic(locationMap, getURI(), problem, element);
		getDiagnostics(problem.getSeverity()).add(diagnostic);
		mark(diagnostic);
		addQuickFixesToQuickFixMap(problem);
	}
	
	public void addProblem(org.reuseware.coconut.fracol.resource.fracol.IFracolProblem problem, int column, int line, int charStart, int charEnd) {
		PositionBasedTextDiagnostic diagnostic = new PositionBasedTextDiagnostic(getURI(), problem, column, line, charStart, charEnd);
		getDiagnostics(problem.getSeverity()).add(diagnostic);
		mark(diagnostic);
		addQuickFixesToQuickFixMap(problem);
	}
	
	protected void addQuickFixesToQuickFixMap(org.reuseware.coconut.fracol.resource.fracol.IFracolProblem problem) {
		java.util.Collection<org.reuseware.coconut.fracol.resource.fracol.IFracolQuickFix> quickFixes = problem.getQuickFixes();
		if (quickFixes != null) {
			for (org.reuseware.coconut.fracol.resource.fracol.IFracolQuickFix quickFix : quickFixes) {
				if (quickFix != null) {
					quickFixMap.put(quickFix.getContextAsString(), quickFix);
				}
			}
		}
	}
	
	@Deprecated	
	public void addError(String message, org.eclipse.emf.ecore.EObject cause) {
		addError(message, org.reuseware.coconut.fracol.resource.fracol.FracolEProblemType.UNKNOWN, cause);
	}
	
	public void addError(String message, org.reuseware.coconut.fracol.resource.fracol.FracolEProblemType type, org.eclipse.emf.ecore.EObject cause) {
		addProblem(new org.reuseware.coconut.fracol.resource.fracol.mopp.FracolProblem(message, type, org.reuseware.coconut.fracol.resource.fracol.FracolEProblemSeverity.ERROR), cause);
	}
	
	@Deprecated	
	public void addWarning(String message, org.eclipse.emf.ecore.EObject cause) {
		addWarning(message, org.reuseware.coconut.fracol.resource.fracol.FracolEProblemType.UNKNOWN, cause);
	}
	
	public void addWarning(String message, org.reuseware.coconut.fracol.resource.fracol.FracolEProblemType type, org.eclipse.emf.ecore.EObject cause) {
		addProblem(new org.reuseware.coconut.fracol.resource.fracol.mopp.FracolProblem(message, type, org.reuseware.coconut.fracol.resource.fracol.FracolEProblemSeverity.WARNING), cause);
	}
	
	protected java.util.List<org.eclipse.emf.ecore.resource.Resource.Diagnostic> getDiagnostics(org.reuseware.coconut.fracol.resource.fracol.FracolEProblemSeverity severity) {
		if (severity == org.reuseware.coconut.fracol.resource.fracol.FracolEProblemSeverity.ERROR) {
			return getErrors();
		} else {
			return getWarnings();
		}
	}
	
	protected java.util.Map<Object, Object> addDefaultLoadOptions(java.util.Map<?, ?> loadOptions) {
		java.util.Map<Object, Object> loadOptionsCopy = org.reuseware.coconut.fracol.resource.fracol.util.FracolMapUtil.copySafelyToObjectToObjectMap(loadOptions);
		// first add static option provider
		loadOptionsCopy.putAll(new org.reuseware.coconut.fracol.resource.fracol.mopp.FracolOptionProvider().getOptions());
		
		// second, add dynamic option providers that are registered via extension
		if (new org.reuseware.coconut.fracol.resource.fracol.util.FracolRuntimeUtil().isEclipsePlatformAvailable()) {
			new org.reuseware.coconut.fracol.resource.fracol.util.FracolEclipseProxy().getDefaultLoadOptionProviderExtensions(loadOptionsCopy);
		}
		return loadOptionsCopy;
	}
	
	/**
	 * Extends the super implementation by clearing all information about element
	 * positions.
	 */
	protected void clearState() {
		// clear concrete syntax information
		resetLocationMap();
		internalURIFragmentMap.clear();
		getErrors().clear();
		getWarnings().clear();
		unmark(org.reuseware.coconut.fracol.resource.fracol.FracolEProblemType.UNKNOWN);
		unmark(org.reuseware.coconut.fracol.resource.fracol.FracolEProblemType.SYNTAX_ERROR);
		unmark(org.reuseware.coconut.fracol.resource.fracol.FracolEProblemType.UNRESOLVED_REFERENCE);
		proxyCounter = 0;
		resolverSwitch = null;
	}
	
	/**
	 * Returns a copy of the contents of this resource wrapped in a list that
	 * propagates changes to the original resource list. Wrapping is required to make
	 * sure that clients which obtain a reference to the list of contents do not
	 * interfere when changing the list.
	 */
	public org.eclipse.emf.common.util.EList<org.eclipse.emf.ecore.EObject> getContents() {
		if (terminateReload) {
			// the contents' state is currently unclear
			return new org.eclipse.emf.common.util.BasicEList<org.eclipse.emf.ecore.EObject>();
		}
		return new org.reuseware.coconut.fracol.resource.fracol.util.FracolCopiedEObjectInternalEList((org.eclipse.emf.ecore.util.InternalEList<org.eclipse.emf.ecore.EObject>) super.getContents());
	}
	
	/**
	 * Returns the raw contents of this resource. In contrast to getContents(), this
	 * methods does not return a copy of the content list, but the original list.
	 */
	public org.eclipse.emf.common.util.EList<org.eclipse.emf.ecore.EObject> getContentsInternal() {
		if (terminateReload) {
			// the contents' state is currently unclear
			return new org.eclipse.emf.common.util.BasicEList<org.eclipse.emf.ecore.EObject>();
		}
		return super.getContents();
	}
	
	/**
	 * Returns all warnings that are associated with this resource.
	 */
	public org.eclipse.emf.common.util.EList<org.eclipse.emf.ecore.resource.Resource.Diagnostic> getWarnings() {
		if (terminateReload) {
			// the contents' state is currently unclear
			return new org.eclipse.emf.common.util.BasicEList<org.eclipse.emf.ecore.resource.Resource.Diagnostic>();
		}
		return new org.reuseware.coconut.fracol.resource.fracol.util.FracolCopiedEList<org.eclipse.emf.ecore.resource.Resource.Diagnostic>(super.getWarnings());
	}
	
	/**
	 * Returns all errors that are associated with this resource.
	 */
	public org.eclipse.emf.common.util.EList<org.eclipse.emf.ecore.resource.Resource.Diagnostic> getErrors() {
		if (terminateReload) {
			// the contents' state is currently unclear
			return new org.eclipse.emf.common.util.BasicEList<org.eclipse.emf.ecore.resource.Resource.Diagnostic>();
		}
		return new org.reuseware.coconut.fracol.resource.fracol.util.FracolCopiedEList<org.eclipse.emf.ecore.resource.Resource.Diagnostic>(super.getErrors());
	}
	
	protected void runValidators(org.eclipse.emf.ecore.EObject root) {
		// check constraints provided by EMF Validator classes
		org.eclipse.emf.common.util.Diagnostic diagnostics = org.eclipse.emf.ecore.util.Diagnostician.INSTANCE.validate(root);
		addDiagnostics(diagnostics, root);
		
		if (new org.reuseware.coconut.fracol.resource.fracol.util.FracolRuntimeUtil().isEclipsePlatformAvailable()) {
			new org.reuseware.coconut.fracol.resource.fracol.util.FracolEclipseProxy().checkEMFValidationConstraints(this, root);
		}
	}
	
	protected void addDiagnostics(org.eclipse.emf.common.util.Diagnostic diagnostics, org.eclipse.emf.ecore.EObject root) {
		org.eclipse.emf.ecore.EObject cause = root;
		java.util.List<?> data = diagnostics.getData();
		if (data != null && data.size() > 0) {
			Object causeObject = data.get(0);
			if (causeObject instanceof org.eclipse.emf.ecore.EObject) {
				cause = (org.eclipse.emf.ecore.EObject) causeObject;
			}
		}
		java.util.List<org.eclipse.emf.common.util.Diagnostic> children = diagnostics.getChildren();
		if (children.size() == 0) {
			if (diagnostics.getSeverity() == org.eclipse.core.runtime.IStatus.ERROR) {
				addError(diagnostics.getMessage(), cause);
			}
			if (diagnostics.getSeverity() == org.eclipse.core.runtime.IStatus.WARNING) {
				addWarning(diagnostics.getMessage(), cause);
			}
		}
		for (org.eclipse.emf.common.util.Diagnostic diagnostic : children) {
			addDiagnostics(diagnostic, root);
		}
	}
	
	public org.reuseware.coconut.fracol.resource.fracol.IFracolQuickFix getQuickFix(String quickFixContext) {
		return quickFixMap.get(quickFixContext);
	}
	
	protected void mark(org.reuseware.coconut.fracol.resource.fracol.IFracolTextDiagnostic diagnostic) {
		org.reuseware.coconut.fracol.resource.fracol.mopp.FracolMarkerHelper markerHelper = getMarkerHelper();
		if (markerHelper != null) {
			markerHelper.mark(this, diagnostic);
		}
	}
	
	protected void unmark(org.eclipse.emf.ecore.EObject cause) {
		org.reuseware.coconut.fracol.resource.fracol.mopp.FracolMarkerHelper markerHelper = getMarkerHelper();
		if (markerHelper != null) {
			markerHelper.unmark(this, cause);
		}
	}
	
	protected void unmark(org.reuseware.coconut.fracol.resource.fracol.FracolEProblemType analysisProblem) {
		org.reuseware.coconut.fracol.resource.fracol.mopp.FracolMarkerHelper markerHelper = getMarkerHelper();
		if (markerHelper != null) {
			markerHelper.unmark(this, analysisProblem);
		}
	}
	
	protected org.reuseware.coconut.fracol.resource.fracol.mopp.FracolMarkerHelper getMarkerHelper() {
		if (isMarkerCreationEnabled() && new org.reuseware.coconut.fracol.resource.fracol.util.FracolRuntimeUtil().isEclipsePlatformAvailable()) {
			if (markerHelper == null) {
				markerHelper = new org.reuseware.coconut.fracol.resource.fracol.mopp.FracolMarkerHelper();
			}
			return markerHelper;
		}
		return null;
	}
	
	public boolean isMarkerCreationEnabled() {
		if (loadOptions == null) {
			return true;
		}
		return !loadOptions.containsKey(org.reuseware.coconut.fracol.resource.fracol.IFracolOptions.DISABLE_CREATING_MARKERS_FOR_PROBLEMS);
	}
	
	protected boolean isLocationMapEnabled() {
		if (loadOptions == null) {
			return true;
		}
		return !loadOptions.containsKey(org.reuseware.coconut.fracol.resource.fracol.IFracolOptions.DISABLE_LOCATION_MAP);
	}
	
	protected boolean isLayoutInformationRecordingEnabled() {
		if (loadOptions == null) {
			return true;
		}
		return !loadOptions.containsKey(org.reuseware.coconut.fracol.resource.fracol.IFracolOptions.DISABLE_LAYOUT_INFORMATION_RECORDING);
	}
	
}
