/**
 * Copyright (c) 2006-2012
 * Software Technology Group, Dresden University of Technology
 * DevBoost GmbH, Berlin, Amtsgericht Charlottenburg, HRB 140026
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Software Technology Group - TU Dresden, Germany;
 *   DevBoost GmbH - Berlin, Germany
 *      - initial API and implementation
 *  
 */
package org.reuseware.coconut.fragment.util;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notifier;

import org.eclipse.emf.common.notify.impl.AdapterFactoryImpl;

import org.eclipse.emf.ecore.EObject;

import org.reuseware.coconut.fragment.*;

/**
 * <!-- begin-user-doc -->
 * The <b>Adapter Factory</b> for the model.
 * It provides an adapter <code>createXXX</code> method for each class of the model.
 * <!-- end-user-doc -->
 * @see org.reuseware.coconut.fragment.FragmentPackage
 * @generated
 */
public class FragmentAdapterFactory extends AdapterFactoryImpl
{
  /**
   * The cached model package.
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @generated
   */
  protected static FragmentPackage modelPackage;

  /**
   * Creates an instance of the adapter factory.
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @generated
   */
  public FragmentAdapterFactory()
  {
    if (modelPackage == null)
    {
      modelPackage = FragmentPackage.eINSTANCE;
    }
  }

  /**
   * Returns whether this factory is applicable for the type of the object.
   * <!-- begin-user-doc -->
   * This implementation returns <code>true</code> if the object is either the model's package or is an instance object of the model.
   * <!-- end-user-doc -->
   * @return whether this factory is applicable for the type of the object.
   * @generated
   */
  @Override
  public boolean isFactoryForType(Object object)
  {
    if (object == modelPackage)
    {
      return true;
    }
    if (object instanceof EObject)
    {
      return ((EObject)object).eClass().getEPackage() == modelPackage;
    }
    return false;
  }

  /**
   * The switch that delegates to the <code>createXXX</code> methods.
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @generated
   */
  protected FragmentSwitch<Adapter> modelSwitch =
    new FragmentSwitch<Adapter>()
    {
      @Override
      public Adapter caseAPMatchGroup(APMatchGroup object)
      {
        return createAPMatchGroupAdapter();
      }
      @Override
      public Adapter caseAPMatch(APMatch object)
      {
        return createAPMatchAdapter();
      }
      @Override
      public Adapter caseAddressablePoint(AddressablePoint object)
      {
        return createAddressablePointAdapter();
      }
      @Override
      public Adapter caseAnchor(Anchor object)
      {
        return createAnchorAdapter();
      }
      @Override
      public Adapter caseComposedFragment(ComposedFragment object)
      {
        return createComposedFragmentAdapter();
      }
      @Override
      public Adapter caseCompositionInterface(CompositionInterface object)
      {
        return createCompositionInterfaceAdapter();
      }
      @Override
      public Adapter caseFragment(Fragment object)
      {
        return createFragmentAdapter();
      }
      @Override
      public Adapter caseHeterogeneousPort(HeterogeneousPort object)
      {
        return createHeterogeneousPortAdapter();
      }
      @Override
      public Adapter caseHomogeneousPort(HomogeneousPort object)
      {
        return createHomogeneousPortAdapter();
      }
      @Override
      public Adapter caseHook(Hook object)
      {
        return createHookAdapter();
      }
      @Override
      public Adapter caseInstantiationSet(InstantiationSet object)
      {
        return createInstantiationSetAdapter();
      }
      @Override
      public Adapter casePhysicalFragment(PhysicalFragment object)
      {
        return createPhysicalFragmentAdapter();
      }
      @Override
      public Adapter casePort(Port object)
      {
        return createPortAdapter();
      }
      @Override
      public Adapter casePrototype(Prototype object)
      {
        return createPrototypeAdapter();
      }
      @Override
      public Adapter caseReferencePoint(ReferencePoint object)
      {
        return createReferencePointAdapter();
      }
      @Override
      public Adapter caseSlot(Slot object)
      {
        return createSlotAdapter();
      }
      @Override
      public Adapter caseValueHook(ValueHook object)
      {
        return createValueHookAdapter();
      }
      @Override
      public Adapter caseValuePrototype(ValuePrototype object)
      {
        return createValuePrototypeAdapter();
      }
      @Override
      public Adapter caseVariationPoint(VariationPoint object)
      {
        return createVariationPointAdapter();
      }
      @Override
      public Adapter defaultCase(EObject object)
      {
        return createEObjectAdapter();
      }
    };

  /**
   * Creates an adapter for the <code>target</code>.
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @param target the object to adapt.
   * @return the adapter for the <code>target</code>.
   * @generated
   */
  @Override
  public Adapter createAdapter(Notifier target)
  {
    return modelSwitch.doSwitch((EObject)target);
  }


  /**
   * Creates a new adapter for an object of class '{@link org.reuseware.coconut.fragment.APMatchGroup <em>AP Match Group</em>}'.
   * <!-- begin-user-doc -->
   * This default implementation returns null so that we can easily ignore cases;
   * it's useful to ignore a case when inheritance will catch all the cases anyway.
   * <!-- end-user-doc -->
   * @return the new adapter.
   * @see org.reuseware.coconut.fragment.APMatchGroup
   * @generated
   */
  public Adapter createAPMatchGroupAdapter()
  {
    return null;
  }

  /**
   * Creates a new adapter for an object of class '{@link org.reuseware.coconut.fragment.APMatch <em>AP Match</em>}'.
   * <!-- begin-user-doc -->
   * This default implementation returns null so that we can easily ignore cases;
   * it's useful to ignore a case when inheritance will catch all the cases anyway.
   * <!-- end-user-doc -->
   * @return the new adapter.
   * @see org.reuseware.coconut.fragment.APMatch
   * @generated
   */
  public Adapter createAPMatchAdapter()
  {
    return null;
  }

  /**
   * Creates a new adapter for an object of class '{@link org.reuseware.coconut.fragment.AddressablePoint <em>Addressable Point</em>}'.
   * <!-- begin-user-doc -->
   * This default implementation returns null so that we can easily ignore cases;
   * it's useful to ignore a case when inheritance will catch all the cases anyway.
   * <!-- end-user-doc -->
   * @return the new adapter.
   * @see org.reuseware.coconut.fragment.AddressablePoint
   * @generated
   */
  public Adapter createAddressablePointAdapter()
  {
    return null;
  }

  /**
   * Creates a new adapter for an object of class '{@link org.reuseware.coconut.fragment.Anchor <em>Anchor</em>}'.
   * <!-- begin-user-doc -->
   * This default implementation returns null so that we can easily ignore cases;
   * it's useful to ignore a case when inheritance will catch all the cases anyway.
   * <!-- end-user-doc -->
   * @return the new adapter.
   * @see org.reuseware.coconut.fragment.Anchor
   * @generated
   */
  public Adapter createAnchorAdapter()
  {
    return null;
  }

  /**
   * Creates a new adapter for an object of class '{@link org.reuseware.coconut.fragment.ComposedFragment <em>Composed Fragment</em>}'.
   * <!-- begin-user-doc -->
   * This default implementation returns null so that we can easily ignore cases;
   * it's useful to ignore a case when inheritance will catch all the cases anyway.
   * <!-- end-user-doc -->
   * @return the new adapter.
   * @see org.reuseware.coconut.fragment.ComposedFragment
   * @generated
   */
  public Adapter createComposedFragmentAdapter()
  {
    return null;
  }

  /**
   * Creates a new adapter for an object of class '{@link org.reuseware.coconut.fragment.CompositionInterface <em>Composition Interface</em>}'.
   * <!-- begin-user-doc -->
   * This default implementation returns null so that we can easily ignore cases;
   * it's useful to ignore a case when inheritance will catch all the cases anyway.
   * <!-- end-user-doc -->
   * @return the new adapter.
   * @see org.reuseware.coconut.fragment.CompositionInterface
   * @generated
   */
  public Adapter createCompositionInterfaceAdapter()
  {
    return null;
  }

  /**
   * Creates a new adapter for an object of class '{@link org.reuseware.coconut.fragment.Fragment <em>Fragment</em>}'.
   * <!-- begin-user-doc -->
   * This default implementation returns null so that we can easily ignore cases;
   * it's useful to ignore a case when inheritance will catch all the cases anyway.
   * <!-- end-user-doc -->
   * @return the new adapter.
   * @see org.reuseware.coconut.fragment.Fragment
   * @generated
   */
  public Adapter createFragmentAdapter()
  {
    return null;
  }

  /**
   * Creates a new adapter for an object of class '{@link org.reuseware.coconut.fragment.HeterogeneousPort <em>Heterogeneous Port</em>}'.
   * <!-- begin-user-doc -->
   * This default implementation returns null so that we can easily ignore cases;
   * it's useful to ignore a case when inheritance will catch all the cases anyway.
   * <!-- end-user-doc -->
   * @return the new adapter.
   * @see org.reuseware.coconut.fragment.HeterogeneousPort
   * @generated
   */
  public Adapter createHeterogeneousPortAdapter()
  {
    return null;
  }

  /**
   * Creates a new adapter for an object of class '{@link org.reuseware.coconut.fragment.HomogeneousPort <em>Homogeneous Port</em>}'.
   * <!-- begin-user-doc -->
   * This default implementation returns null so that we can easily ignore cases;
   * it's useful to ignore a case when inheritance will catch all the cases anyway.
   * <!-- end-user-doc -->
   * @return the new adapter.
   * @see org.reuseware.coconut.fragment.HomogeneousPort
   * @generated
   */
  public Adapter createHomogeneousPortAdapter()
  {
    return null;
  }

  /**
   * Creates a new adapter for an object of class '{@link org.reuseware.coconut.fragment.Hook <em>Hook</em>}'.
   * <!-- begin-user-doc -->
   * This default implementation returns null so that we can easily ignore cases;
   * it's useful to ignore a case when inheritance will catch all the cases anyway.
   * <!-- end-user-doc -->
   * @return the new adapter.
   * @see org.reuseware.coconut.fragment.Hook
   * @generated
   */
  public Adapter createHookAdapter()
  {
    return null;
  }

  /**
   * Creates a new adapter for an object of class '{@link org.reuseware.coconut.fragment.InstantiationSet <em>Instantiation Set</em>}'.
   * <!-- begin-user-doc -->
   * This default implementation returns null so that we can easily ignore cases;
   * it's useful to ignore a case when inheritance will catch all the cases anyway.
   * <!-- end-user-doc -->
   * @return the new adapter.
   * @see org.reuseware.coconut.fragment.InstantiationSet
   * @generated
   */
  public Adapter createInstantiationSetAdapter()
  {
    return null;
  }

  /**
   * Creates a new adapter for an object of class '{@link org.reuseware.coconut.fragment.PhysicalFragment <em>Physical Fragment</em>}'.
   * <!-- begin-user-doc -->
   * This default implementation returns null so that we can easily ignore cases;
   * it's useful to ignore a case when inheritance will catch all the cases anyway.
   * <!-- end-user-doc -->
   * @return the new adapter.
   * @see org.reuseware.coconut.fragment.PhysicalFragment
   * @generated
   */
  public Adapter createPhysicalFragmentAdapter()
  {
    return null;
  }

  /**
   * Creates a new adapter for an object of class '{@link org.reuseware.coconut.fragment.Port <em>Port</em>}'.
   * <!-- begin-user-doc -->
   * This default implementation returns null so that we can easily ignore cases;
   * it's useful to ignore a case when inheritance will catch all the cases anyway.
   * <!-- end-user-doc -->
   * @return the new adapter.
   * @see org.reuseware.coconut.fragment.Port
   * @generated
   */
  public Adapter createPortAdapter()
  {
    return null;
  }

  /**
   * Creates a new adapter for an object of class '{@link org.reuseware.coconut.fragment.Prototype <em>Prototype</em>}'.
   * <!-- begin-user-doc -->
   * This default implementation returns null so that we can easily ignore cases;
   * it's useful to ignore a case when inheritance will catch all the cases anyway.
   * <!-- end-user-doc -->
   * @return the new adapter.
   * @see org.reuseware.coconut.fragment.Prototype
   * @generated
   */
  public Adapter createPrototypeAdapter()
  {
    return null;
  }

  /**
   * Creates a new adapter for an object of class '{@link org.reuseware.coconut.fragment.ReferencePoint <em>Reference Point</em>}'.
   * <!-- begin-user-doc -->
   * This default implementation returns null so that we can easily ignore cases;
   * it's useful to ignore a case when inheritance will catch all the cases anyway.
   * <!-- end-user-doc -->
   * @return the new adapter.
   * @see org.reuseware.coconut.fragment.ReferencePoint
   * @generated
   */
  public Adapter createReferencePointAdapter()
  {
    return null;
  }

  /**
   * Creates a new adapter for an object of class '{@link org.reuseware.coconut.fragment.Slot <em>Slot</em>}'.
   * <!-- begin-user-doc -->
   * This default implementation returns null so that we can easily ignore cases;
   * it's useful to ignore a case when inheritance will catch all the cases anyway.
   * <!-- end-user-doc -->
   * @return the new adapter.
   * @see org.reuseware.coconut.fragment.Slot
   * @generated
   */
  public Adapter createSlotAdapter()
  {
    return null;
  }

  /**
   * Creates a new adapter for an object of class '{@link org.reuseware.coconut.fragment.ValueHook <em>Value Hook</em>}'.
   * <!-- begin-user-doc -->
   * This default implementation returns null so that we can easily ignore cases;
   * it's useful to ignore a case when inheritance will catch all the cases anyway.
   * <!-- end-user-doc -->
   * @return the new adapter.
   * @see org.reuseware.coconut.fragment.ValueHook
   * @generated
   */
  public Adapter createValueHookAdapter()
  {
    return null;
  }

  /**
   * Creates a new adapter for an object of class '{@link org.reuseware.coconut.fragment.ValuePrototype <em>Value Prototype</em>}'.
   * <!-- begin-user-doc -->
   * This default implementation returns null so that we can easily ignore cases;
   * it's useful to ignore a case when inheritance will catch all the cases anyway.
   * <!-- end-user-doc -->
   * @return the new adapter.
   * @see org.reuseware.coconut.fragment.ValuePrototype
   * @generated
   */
  public Adapter createValuePrototypeAdapter()
  {
    return null;
  }

  /**
   * Creates a new adapter for an object of class '{@link org.reuseware.coconut.fragment.VariationPoint <em>Variation Point</em>}'.
   * <!-- begin-user-doc -->
   * This default implementation returns null so that we can easily ignore cases;
   * it's useful to ignore a case when inheritance will catch all the cases anyway.
   * <!-- end-user-doc -->
   * @return the new adapter.
   * @see org.reuseware.coconut.fragment.VariationPoint
   * @generated
   */
  public Adapter createVariationPointAdapter()
  {
    return null;
  }

  /**
   * Creates a new adapter for the default case.
   * <!-- begin-user-doc -->
   * This default implementation returns null.
   * <!-- end-user-doc -->
   * @return the new adapter.
   * @generated
   */
  public Adapter createEObjectAdapter()
  {
    return null;
  }

} //FragmentAdapterFactory
