/**
 * Copyright (c) 2006-2012
 * Software Technology Group, Dresden University of Technology
 * DevBoost GmbH, Berlin, Amtsgericht Charlottenburg, HRB 140026
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Software Technology Group - TU Dresden, Germany;
 *   DevBoost GmbH - Berlin, Germany
 *      - initial API and implementation
 *  
 */
package org.reuseware.coconut.fragment.util;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;

import org.eclipse.emf.ecore.util.Switch;

import org.reuseware.coconut.fragment.*;

/**
 * <!-- begin-user-doc -->
 * The <b>Switch</b> for the model's inheritance hierarchy.
 * It supports the call {@link #doSwitch(EObject) doSwitch(object)}
 * to invoke the <code>caseXXX</code> method for each class of the model,
 * starting with the actual class of the object
 * and proceeding up the inheritance hierarchy
 * until a non-null result is returned,
 * which is the result of the switch.
 * <!-- end-user-doc -->
 * @see org.reuseware.coconut.fragment.FragmentPackage
 * @generated
 */
public class FragmentSwitch<T> extends Switch<T>
{
  /**
   * The cached model package
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @generated
   */
  protected static FragmentPackage modelPackage;

  /**
   * Creates an instance of the switch.
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @generated
   */
  public FragmentSwitch()
  {
    if (modelPackage == null)
    {
      modelPackage = FragmentPackage.eINSTANCE;
    }
  }

  /**
   * Checks whether this is a switch for the given package.
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @parameter ePackage the package in question.
   * @return whether this is a switch for the given package.
   * @generated
   */
  @Override
  protected boolean isSwitchFor(EPackage ePackage)
  {
    return ePackage == modelPackage;
  }

  /**
   * Calls <code>caseXXX</code> for each class of the model until one returns a non null result; it yields that result.
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @return the first non-null result returned by a <code>caseXXX</code> call.
   * @generated
   */
  @Override
  protected T doSwitch(int classifierID, EObject theEObject)
  {
    switch (classifierID)
    {
      case FragmentPackage.AP_MATCH_GROUP:
      {
        APMatchGroup apMatchGroup = (APMatchGroup)theEObject;
        T result = caseAPMatchGroup(apMatchGroup);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case FragmentPackage.AP_MATCH:
      {
        APMatch apMatch = (APMatch)theEObject;
        T result = caseAPMatch(apMatch);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case FragmentPackage.ADDRESSABLE_POINT:
      {
        AddressablePoint addressablePoint = (AddressablePoint)theEObject;
        T result = caseAddressablePoint(addressablePoint);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case FragmentPackage.ANCHOR:
      {
        Anchor anchor = (Anchor)theEObject;
        T result = caseAnchor(anchor);
        if (result == null) result = caseReferencePoint(anchor);
        if (result == null) result = caseAddressablePoint(anchor);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case FragmentPackage.COMPOSED_FRAGMENT:
      {
        ComposedFragment composedFragment = (ComposedFragment)theEObject;
        T result = caseComposedFragment(composedFragment);
        if (result == null) result = caseFragment(composedFragment);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case FragmentPackage.COMPOSITION_INTERFACE:
      {
        CompositionInterface compositionInterface = (CompositionInterface)theEObject;
        T result = caseCompositionInterface(compositionInterface);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case FragmentPackage.FRAGMENT:
      {
        Fragment fragment = (Fragment)theEObject;
        T result = caseFragment(fragment);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case FragmentPackage.HETEROGENEOUS_PORT:
      {
        HeterogeneousPort heterogeneousPort = (HeterogeneousPort)theEObject;
        T result = caseHeterogeneousPort(heterogeneousPort);
        if (result == null) result = casePort(heterogeneousPort);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case FragmentPackage.HOMOGENEOUS_PORT:
      {
        HomogeneousPort homogeneousPort = (HomogeneousPort)theEObject;
        T result = caseHomogeneousPort(homogeneousPort);
        if (result == null) result = casePort(homogeneousPort);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case FragmentPackage.HOOK:
      {
        Hook hook = (Hook)theEObject;
        T result = caseHook(hook);
        if (result == null) result = caseVariationPoint(hook);
        if (result == null) result = caseAddressablePoint(hook);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case FragmentPackage.INSTANTIATION_SET:
      {
        InstantiationSet instantiationSet = (InstantiationSet)theEObject;
        T result = caseInstantiationSet(instantiationSet);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case FragmentPackage.PHYSICAL_FRAGMENT:
      {
        PhysicalFragment physicalFragment = (PhysicalFragment)theEObject;
        T result = casePhysicalFragment(physicalFragment);
        if (result == null) result = caseFragment(physicalFragment);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case FragmentPackage.PORT:
      {
        Port port = (Port)theEObject;
        T result = casePort(port);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case FragmentPackage.PROTOTYPE:
      {
        Prototype prototype = (Prototype)theEObject;
        T result = casePrototype(prototype);
        if (result == null) result = caseReferencePoint(prototype);
        if (result == null) result = caseAddressablePoint(prototype);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case FragmentPackage.REFERENCE_POINT:
      {
        ReferencePoint referencePoint = (ReferencePoint)theEObject;
        T result = caseReferencePoint(referencePoint);
        if (result == null) result = caseAddressablePoint(referencePoint);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case FragmentPackage.SLOT:
      {
        Slot slot = (Slot)theEObject;
        T result = caseSlot(slot);
        if (result == null) result = caseVariationPoint(slot);
        if (result == null) result = caseAddressablePoint(slot);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case FragmentPackage.VALUE_HOOK:
      {
        ValueHook valueHook = (ValueHook)theEObject;
        T result = caseValueHook(valueHook);
        if (result == null) result = caseVariationPoint(valueHook);
        if (result == null) result = caseAddressablePoint(valueHook);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case FragmentPackage.VALUE_PROTOTYPE:
      {
        ValuePrototype valuePrototype = (ValuePrototype)theEObject;
        T result = caseValuePrototype(valuePrototype);
        if (result == null) result = caseReferencePoint(valuePrototype);
        if (result == null) result = caseAddressablePoint(valuePrototype);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case FragmentPackage.VARIATION_POINT:
      {
        VariationPoint variationPoint = (VariationPoint)theEObject;
        T result = caseVariationPoint(variationPoint);
        if (result == null) result = caseAddressablePoint(variationPoint);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      default: return defaultCase(theEObject);
    }
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>AP Match Group</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>AP Match Group</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseAPMatchGroup(APMatchGroup object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>AP Match</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>AP Match</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseAPMatch(APMatch object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Addressable Point</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Addressable Point</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseAddressablePoint(AddressablePoint object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Anchor</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Anchor</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseAnchor(Anchor object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Composed Fragment</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Composed Fragment</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseComposedFragment(ComposedFragment object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Composition Interface</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Composition Interface</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseCompositionInterface(CompositionInterface object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Fragment</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Fragment</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseFragment(Fragment object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Heterogeneous Port</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Heterogeneous Port</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseHeterogeneousPort(HeterogeneousPort object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Homogeneous Port</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Homogeneous Port</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseHomogeneousPort(HomogeneousPort object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Hook</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Hook</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseHook(Hook object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Instantiation Set</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Instantiation Set</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseInstantiationSet(InstantiationSet object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Physical Fragment</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Physical Fragment</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T casePhysicalFragment(PhysicalFragment object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Port</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Port</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T casePort(Port object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Prototype</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Prototype</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T casePrototype(Prototype object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Reference Point</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Reference Point</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseReferencePoint(ReferencePoint object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Slot</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Slot</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseSlot(Slot object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Value Hook</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Value Hook</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseValueHook(ValueHook object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Value Prototype</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Value Prototype</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseValuePrototype(ValuePrototype object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Variation Point</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Variation Point</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseVariationPoint(VariationPoint object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>EObject</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch, but this is the last case anyway.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>EObject</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject)
   * @generated
   */
  @Override
  public T defaultCase(EObject object)
  {
    return null;
  }

} //FragmentSwitch
