/*******************************************************************************
 * Copyright (c) 2006-2012
 * Software Technology Group, Dresden University of Technology
 * DevBoost GmbH, Berlin, Amtsgericht Charlottenburg, HRB 140026
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Software Technology Group - TU Dresden, Germany;
 *   DevBoost GmbH - Berlin, Germany
 *      - initial API and implementation
 ******************************************************************************/

package org.reuseware.coconut.reuseextensionactivator.resource.rexactivator.util;

/**
 * This class provides basic infrastructure to interpret models. To implement
 * concrete interpreters, subclass this abstract interpreter and override the
 * interprete_* methods. The interpretation can be customized by binding the two
 * type parameters (ResultType, ContextType). The former is returned by all
 * interprete_* methods, while the latter is passed from method to method while
 * traversing the model. The concrete traversal strategy can also be exchanged.
 * One can use a static traversal strategy by pushing all objects to interpret on
 * the interpretation stack (using addObjectToInterprete()) before calling
 * interprete(). Alternatively, the traversal strategy can be dynamic by pushing
 * objects on the interpretation stack during interpretation.
 */
public class AbstractRex_activatorInterpreter<ResultType, ContextType> {
	
	private java.util.Stack<org.eclipse.emf.ecore.EObject> interpretationStack = new java.util.Stack<org.eclipse.emf.ecore.EObject>();
	private java.util.List<org.reuseware.coconut.reuseextensionactivator.resource.rexactivator.IRex_activatorInterpreterListener> listeners = new java.util.ArrayList<org.reuseware.coconut.reuseextensionactivator.resource.rexactivator.IRex_activatorInterpreterListener>();
	private org.eclipse.emf.ecore.EObject nextObjectToInterprete;
	private Object currentContext;
	
	public ResultType interprete(ContextType context) {
		ResultType result = null;
		org.eclipse.emf.ecore.EObject next = null;
		currentContext = context;
		while (!interpretationStack.empty()) {
			try {
				next = interpretationStack.pop();
			} catch (java.util.EmptyStackException ese) {
				// this can happen when the interpreter was terminated between the call to empty()
				// and pop()
				break;
			}
			nextObjectToInterprete = next;
			notifyListeners(next);
			result = interprete(next, context);
			if (!continueInterpretation(context, result)) {
				break;
			}
		}
		currentContext = null;
		return result;
	}
	
	/**
	 * Override this method to stop the overall interpretation depending on the result
	 * of the interpretation of a single model elements.
	 */
	public boolean continueInterpretation(ContextType context, ResultType result) {
		return true;
	}
	
	public ResultType interprete(org.eclipse.emf.ecore.EObject object, ContextType context) {
		ResultType result = null;
		if (object instanceof org.reuseware.coconut.reuseextensionactivator.Argument) {
			result = interprete_org_reuseware_coconut_reuseextensionactivator_Argument((org.reuseware.coconut.reuseextensionactivator.Argument) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextensionactivator.ComponentModelSpecificationActivator) {
			result = interprete_org_reuseware_coconut_reuseextensionactivator_ComponentModelSpecificationActivator((org.reuseware.coconut.reuseextensionactivator.ComponentModelSpecificationActivator) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextensionactivator.CompositionLanguageSpecificationActivator) {
			result = interprete_org_reuseware_coconut_reuseextensionactivator_CompositionLanguageSpecificationActivator((org.reuseware.coconut.reuseextensionactivator.CompositionLanguageSpecificationActivator) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextensionactivator.CompositionLanguageSyntaxSpecificationActivator) {
			result = interprete_org_reuseware_coconut_reuseextensionactivator_CompositionLanguageSyntaxSpecificationActivator((org.reuseware.coconut.reuseextensionactivator.CompositionLanguageSyntaxSpecificationActivator) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextensionactivator.ReuseExtensionActivatorList) {
			result = interprete_org_reuseware_coconut_reuseextensionactivator_ReuseExtensionActivatorList((org.reuseware.coconut.reuseextensionactivator.ReuseExtensionActivatorList) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextensionactivator.ReuseExtensionActivator) {
			result = interprete_org_reuseware_coconut_reuseextensionactivator_ReuseExtensionActivator((org.reuseware.coconut.reuseextensionactivator.ReuseExtensionActivator) object, context);
		}
		if (result != null) {
			return result;
		}
		return result;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextensionactivator_Argument(org.reuseware.coconut.reuseextensionactivator.Argument argument, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextensionactivator_ComponentModelSpecificationActivator(org.reuseware.coconut.reuseextensionactivator.ComponentModelSpecificationActivator componentModelSpecificationActivator, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextensionactivator_CompositionLanguageSpecificationActivator(org.reuseware.coconut.reuseextensionactivator.CompositionLanguageSpecificationActivator compositionLanguageSpecificationActivator, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextensionactivator_CompositionLanguageSyntaxSpecificationActivator(org.reuseware.coconut.reuseextensionactivator.CompositionLanguageSyntaxSpecificationActivator compositionLanguageSyntaxSpecificationActivator, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextensionactivator_ReuseExtensionActivatorList(org.reuseware.coconut.reuseextensionactivator.ReuseExtensionActivatorList reuseExtensionActivatorList, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextensionactivator_ReuseExtensionActivator(org.reuseware.coconut.reuseextensionactivator.ReuseExtensionActivator reuseExtensionActivator, ContextType context) {
		return null;
	}
	
	private void notifyListeners(org.eclipse.emf.ecore.EObject element) {
		for (org.reuseware.coconut.reuseextensionactivator.resource.rexactivator.IRex_activatorInterpreterListener listener : listeners) {
			listener.handleInterpreteObject(element);
		}
	}
	
	/**
	 * Adds the given object to the interpretation stack. Attention: Objects that are
	 * added first, are interpret last.
	 */
	public void addObjectToInterprete(org.eclipse.emf.ecore.EObject object) {
		interpretationStack.push(object);
	}
	
	/**
	 * Adds the given collection of objects to the interpretation stack. Attention:
	 * Collections that are added first, are interpret last.
	 */
	public void addObjectsToInterprete(java.util.Collection<? extends org.eclipse.emf.ecore.EObject> objects) {
		for (org.eclipse.emf.ecore.EObject object : objects) {
			addObjectToInterprete(object);
		}
	}
	
	/**
	 * Adds the given collection of objects in reverse order to the interpretation
	 * stack.
	 */
	public void addObjectsToInterpreteInReverseOrder(java.util.Collection<? extends org.eclipse.emf.ecore.EObject> objects) {
		java.util.List<org.eclipse.emf.ecore.EObject> reverse = new java.util.ArrayList<org.eclipse.emf.ecore.EObject>(objects.size());
		reverse.addAll(objects);
		java.util.Collections.reverse(reverse);
		addObjectsToInterprete(reverse);
	}
	
	/**
	 * Adds the given object and all its children to the interpretation stack such
	 * that they are interpret in top down order.
	 */
	public void addObjectTreeToInterpreteTopDown(org.eclipse.emf.ecore.EObject root) {
		java.util.List<org.eclipse.emf.ecore.EObject> objects = new java.util.ArrayList<org.eclipse.emf.ecore.EObject>();
		objects.add(root);
		java.util.Iterator<org.eclipse.emf.ecore.EObject> it = root.eAllContents();
		while (it.hasNext()) {
			org.eclipse.emf.ecore.EObject eObject = (org.eclipse.emf.ecore.EObject) it.next();
			objects.add(eObject);
		}
		addObjectsToInterpreteInReverseOrder(objects);
	}
	
	public void addListener(org.reuseware.coconut.reuseextensionactivator.resource.rexactivator.IRex_activatorInterpreterListener newListener) {
		listeners.add(newListener);
	}
	
	public boolean removeListener(org.reuseware.coconut.reuseextensionactivator.resource.rexactivator.IRex_activatorInterpreterListener listener) {
		return listeners.remove(listener);
	}
	
	public org.eclipse.emf.ecore.EObject getNextObjectToInterprete() {
		return nextObjectToInterprete;
	}
	
	public java.util.Stack<org.eclipse.emf.ecore.EObject> getInterpretationStack() {
		return interpretationStack;
	}
	
	public void terminate() {
		interpretationStack.clear();
	}
	
	public Object getCurrentContext() {
		return currentContext;
	}
	
}
